/*
 Standalone Temperature Logger - Test on Arduino
 Version: 1.0
 Author: electricteardown (http://www.insidegadgets.com)
 Created: 9/01/2011
 Last Modified: 9/01/2011
 
 Used to monitor temperature every x minutes/hours which is saved 
 to the built in EEPROM. 
 
 */

#include <math.h>
#include <EEPROM.h>

int ledPin = 13;
int thermistorPin = 0;
int buttonPin = 2;

int functionSelect = 0;
int dataCount = 1;
int delayTime = -1;

void setup() {
  pinMode(ledPin, OUTPUT);
  pinMode(buttonPin, INPUT);    
}

void loop() {
  
  // Wait for input
  if (functionSelect == 0) {
    
    // Read delay time
    if (delayTime == -1) {
      delayTime = EEPROM.read(0);
      if (delayTime > 10) { // Not a normal value
        delayTime = 0;
      }
    }
    
    // Check button press, if still pressed after 500ms, check again
    // after 1500ms, if still pressed go to configuration
    int buttonState = digitalRead(buttonPin);
    if (buttonState == HIGH) {
      delay(500);
      buttonState = digitalRead(buttonPin);
      if (buttonState == true) {
        delay(1500);
        buttonState = digitalRead(buttonPin);
        if (buttonState == HIGH) {
          functionSelect = 2;
          delayTime = -1;
          digitalWrite(ledPin, HIGH);
          delay(2000);
          digitalWrite(ledPin, LOW);
        }
      }
      else {
        functionSelect = 1;
        for (int x = 0; x < 3; x++) {
          digitalWrite(ledPin, HIGH);
          delay(500);
          digitalWrite(ledPin, LOW);
          delay(500);
        }
      }
    }
  }
  
  // Log Temperature
  if (functionSelect == 1) {
    int tempValue = int(Thermistor(analogRead(thermistorPin)));
    
    // Calculations to store -40C to 125C int to a single byte
    byte tempStore = tempValue + 130;
    if (tempValue <= -41) { // Temp is over min temp accepted
      tempStore = 90;
    }
    if (tempValue >= 126) { // Temp is over max temp accepted
      tempStore = 255;
    }
    
    // Write to EEPROM
    if (dataCount < 512) {
      EEPROM.write(dataCount, tempStore);
      dataCount++;
      if (dataCount < 512) {
        EEPROM.write(dataCount, 0); // So we know where to stop when extracting data
      }
    }
        
    // Calculate the delay time
    if (delayTime == 0) {
      delay(60000);
    }
    if (delayTime >= 1 && delayTime < 4) {
      delay((delayTime * 5) * 60000);
    }
    if (delayTime >= 4 && delayTime < 6) {
      delay(((delayTime-3) * 30) * 60000);
    }
    if (delayTime >= 6 && delayTime < 8) {
      delay(((delayTime-5) * 120) * 60000);
    }
    if (delayTime == 8) {
      delay(8 * 60 * 60000);
    }
    if (delayTime == 9) {
      delay(12 * 60 * 60000);
    }
    if (delayTime == 10) {
      delay(24 * 60 * 60000);
    }
  }
  
  // Configure the delay time
  if (functionSelect == 2) {
    int buttonState = digitalRead(buttonPin);
    if (buttonState == HIGH) {
      delay(500);
      int buttonPressed = false;
      buttonState = digitalRead(buttonPin);
      if (buttonState == HIGH && delayTime >= 0) {
        delay(1500);
        buttonState = digitalRead(buttonPin);
        if (buttonState == HIGH) {
          functionSelect = 0;
          EEPROM.write(0, delayTime);
          for (int x = 0; x < 3; x++) {
            digitalWrite(ledPin, HIGH);
            delay(500);
            digitalWrite(ledPin, LOW);
            delay(500);
          }
        }
        else {
          buttonPressed = true;
        }
      }
      if (buttonState == LOW || buttonPressed == true) {
        delayTime++;
        digitalWrite(ledPin, HIGH);
        delay(500);
        digitalWrite(ledPin, LOW);
        
        if (delayTime > 10) { // Too many presses
          delayTime = -1;
          digitalWrite(ledPin, HIGH);
          delay(1500);
          digitalWrite(ledPin, LOW);
        }
      }
    }
  }
}

// Taken from http://www.arduino.cc/playground/ComponentLib/Thermistor2
double Thermistor(int RawADC) {
  double Temp;
  Temp = log(((10240000/RawADC) - 10000));
  Temp = 1 / (0.001129148 + (0.000234125 * Temp) + (0.0000000876741 * Temp * Temp * Temp));
  Temp = Temp - 273.15;            // Convert Kelvin to Celcius
  return Temp;
}

